# When this class is initialized in fvApp, the constructor will create a
# ::fvPref namespace with various variable names

class fvPreferences {

   # Create and initialize fvPref namespace
   constructor { args } {}
   destructor { }

   public {
      method save  { }
      method edit  { {page "fv"} }
      method close { }
      method raisePage { page }
      method _setGraphSize { menu {val ""} }
   }

   private {
      method _read {}

      method _buildPage_App      { w }
      method _buildPage_Keywords { w }
      method _buildPage_Tables   { w }
      method _buildPage_Graphs   { w }
      method _buildPage_Colors   { w }
      method _buildPage_Library  { w }

      method _getGraphSize { }
      method _changeColor  { type }
      method _changeWindowColors { win type }

      variable note
      variable _FVRC
      variable _padXAmnt 10
      variable _padYAmnt 3
      variable _tabLabels [list App Keywords Tables Graphs Colors]
   }
}

body fvPreferences::constructor { args } {
   global tcl_platform env FITSVIEWER_LIBRARY
   global g_titleFont

   switch $tcl_platform(platform) {
      "macintosh" {
         set _FVRC [file join $env(PREF_FOLDER) "fv Preferences"]
      }
      "windows" {
         set  FITSVIEWER_LIBRARY $env(FITSVIEWER_LIBRARY)
         set _FVRC [file join $FITSVIEWER_LIBRARY "fv.ini"]
         # Pan Chai: uncomment when XPA is available in Windows
         # lappend _tabLabels Library
      }
      default {
         set _FVRC "~/.fvrc"
      }
   }

   option add *fvPreference.font g_titleFont
   # Create and initialize fvPref namespace

   namespace eval ::fvPref { 
      variable globalBgColor        #cccccc
      variable globalFgColor        black  
      variable activeBgColor        #eeeeee
      variable activeFgColor        black  
      # variable checkBBgColor        #ff3366
      variable checkBBgColor        red
      variable imgDisplayer         POW
      variable ifWriteHisKey        1
      variable ifDispFitsOnly       1
      variable ifWCSInfo            1
      variable ifProtectedKeys      1
      variable ifAutoReformatKeys   1
      variable ifLeftJustifyString  1
      variable ifAutoUpdateChecksum 0
      variable ifUseManager         1
      variable ifAutoPlotPrimary    0
      variable xpaLibPath           
      variable ds9LibPath           
      variable winManagement        Keep
      variable graphDispSize        [list 300 300]
      variable webModulesURLsHead   "http://heasarc.gsfc.nasa.gov/hera/"
   }

   # Override some defaults based on enviroment

   if { [winfo screenheight .]>850 } {
      set fvPref::graphDispSize [list 500 500]
   }

   set tmpBg [option get . background Background]
   if { $tmpBg != ""} {
      set fvPref::globalBgColor $tmpBg
   }
   set tmpFg [option get . foreground Foreground]
   if { $tmpFg != ""} {
      set fvPref::globalFgColor $tmpFg
   }

   if { $tcl_platform(platform)=="macintosh" } {
      set fvPref::ifUseManager 0
      set fvPref::globalBgColor "#eeeeee"
      set fvPref::activeBgColor "#cccccc"
   }

   if [info exists env(HERA_SERVER) ] {
      set fvPref::webModulesURLsHead "http://$env(HERA_SERVER)/hera/"
   } 
      
   #  Read in the preferences file

   _read
}

body fvPreferences::destructor { } {
   if { [winfo exists .fvpref] } {
      destroy .fvpref
   }
}


body fvPreferences::edit { {page "fv"} } {
   global tcl_platform
   global g_titleFont
   
   if { [winfo exists .fvpref] } {
      if {$tcl_platform(platform)!= "windows"} { 
        .fvpref activate
        Notebook:raise [.fvpref childsite].notebook $page
        # since the notebook does not actual destroy, we need this line to update graph size
        _setGraphSize [.fvpref childsite].notebook.f3.graphsize $fvPref::graphDispSize
      } else {
        focus .fvpref
        raise .fvpref        
        Notebook:raise .fvpref.notebook $page
      }        
      return
   }

   ###########
   # Setup Dialog Box
   ###########
   if { $tcl_platform(platform) != "windows" } {  
    ::iwidgets::dialogshell .fvpref -title "fv: Preferences" -modality "none"
    .fvpref add Save -text "Save" -command [code $this save] -font g_titleFont
    .fvpref add Help -text "Help" -command "hhelp preferences" -font g_titleFont
    .fvpref add Exit -text "Exit" -command [code $this close] -font g_titleFont
    .fvpref default Exit
   } else {
     toplevel .fvpref -class Dialog
     wm  title .fvpref "fv: Preferences"
     frame .fvpref.buts
     button .fvpref.buts.save -text Save -command [code $this save] -font g_titleFont
     button .fvpref.buts.help -text Help -command "hhelp preferences" -font g_titleFont
     button .fvpref.buts.exit -text Exit -command [code $this close] -font g_titleFont
     pack .fvpref.buts.save -side left -padx 30 -pady 4          
     pack .fvpref.buts.help -side left -padx 30  -pady 4        
     pack .fvpref.buts.exit -side right -padx 30 -pady 4
                
   }

   bind .fvpref <<CloseWindow>> [code $this close]

   ###########
   # Setup Tabbed Notebook Widget
   ###########

   if {$tcl_platform(platform) != "windows" } {   
      set note "[.fvpref childsite].notebook"
   } else {
      set note .fvpref.notebook
   } 

   Notebook:create $note -pages $_tabLabels -pad 8

   foreach p $_tabLabels {
      eval _buildPage_$p [Notebook:frame $note $p]
   }

   pack $note -fill both -expand 1   
    
   ###########
   # Display dialog
   ###########

   Notebook:raise $note $page
   Notebook:raise.page $note [lsearch $_tabLabels "Graphs"]
   Notebook:resize $note
   if {$tcl_platform(platform) != "windows" } {    
      .fvpref activate
   } else {
      pack .fvpref.buts 
   }
}

body fvPreferences::raisePage { page } {

   Notebook:raise.page $note [lsearch $_tabLabels $page]
   Notebook:resize $note
     
}

body fvPreferences::close { } {
   global tcl_platform
   if { [winfo exists .fvpref] } {
      if {$tcl_platform(platform) == "windows" } { 
        destroy .fvpref
      } else {
        .fvpref deactivate
      }
   }
   checkForExit
}


body fvPreferences::_read { } {
   global tcl_platform

   if { ![catch { set rcFile [open $_FVRC r] } ] } {
      while { [gets $rcFile line] != -1} {
         set entry [lindex $line 0]
         set value [lindex $line 1]
         switch -glob -- $entry {
            "Background" {	
               set fvPref::globalBgColor $value
            }
            "Foreground" {
               set fvPref::globalFgColor $value
            }
            "ActiveBackground" {
               set fvPref::activeBgColor $value
            }
            "ActiveForeground" {
               set fvPref::activeFgColor $value
            }
            "CheckButtonColor" {
               set fvPref::checkBBgColor $value
            }
            "ImageDisplayer" {
               set fvPref::imgDisplayer $value
            }
            "IfWriteHisKey" {
               set fvPref::ifWriteHisKey $value
            } 
            "IfDispFitsOnly" {
               set fvPref::ifDispFitsOnly $value
            } 	
            "IfWCSInfo" {
               set fvPref::ifWCSInfo $value
            }
            "IfProtectedKeys" {
               set fvPref::ifProtectedKeys $value
            }
            "IfAutoReformatKeys" {
               set fvPref::ifAutoReformatKeys $value
            }
            "WinManagement" {
               set fvPref::winManagement $value	
            }		
            "IfAutoUpdateChecksum" {
               set fvPref::ifAutoUpdateChecksum $value
            }	
            "IfLeftJustifyString" {
               set fvPref::ifLeftJustifyString $value
            }
            "IfUseDesktopManager" {
               if { $tcl_platform(platform) != "macintosh" } {
                  set fvPref::ifUseManager $value
               }
            }
            "IfAutoPlotPrimary" {
               set fvPref::ifAutoPlotPrimary $value
            }
            "GraphDisplaySize" {
               set fvPref::graphDispSize $value
            }
            "xpaLibPath" {
               set fvPref::xpaLibPath "$value"
            }
            "ds9LibPath" {
               set fvPref::ds9LibPath "$value"
            }
            "#*" {
            }
            default {
               set fvPref::$entry $value
            }
         }
      }			 
      ::close $rcFile
   }

   option add *Background          $fvPref::globalBgColor
   option add *Foreground          $fvPref::globalFgColor
   option add *HighlightBackground $fvPref::globalBgColor
   option add *activeForeground    $fvPref::activeFgColor
   option add *activeBackground    $fvPref::activeBgColor
   option add *selectForeground    $fvPref::activeFgColor
   option add *selectBackground    $fvPref::activeBgColor
   option add *selectColor         $fvPref::checkBBgColor

}

body fvPreferences::save { } {
    
   global tcl_platform
# check if _FVRC file is there
   catch { [file delete -force $_FVRC]} 

   if { [catch {set rcFile [open $_FVRC w]} err] == 1} {
      error "Unable to open $_FVRC to write"
      return 
   }

   puts $rcFile "\# Please don't change this file. It's auto generated by fv"
   puts $rcFile "Background           $fvPref::globalBgColor"
   puts $rcFile "Foreground           $fvPref::globalFgColor"
   puts $rcFile "ActiveBackground     $fvPref::activeBgColor"
   puts $rcFile "ActiveForeground     $fvPref::activeFgColor"
   puts $rcFile "CheckButtonColor     $fvPref::checkBBgColor"
   puts $rcFile "ImageDisplayer       $fvPref::imgDisplayer"
   puts $rcFile "IfWriteHisKey        $fvPref::ifWriteHisKey"
   puts $rcFile "IfDispFitsOnly       $fvPref::ifDispFitsOnly"
   puts $rcFile "WinManagement        $fvPref::winManagement"
   puts $rcFile "IfWCSInfo            $fvPref::ifWCSInfo"
   puts $rcFile "IfProtectedKeys      $fvPref::ifProtectedKeys"
   puts $rcFile "IfAutoReformatKeys   $fvPref::ifAutoReformatKeys"
   puts $rcFile "IfAutoUpdateChecksum $fvPref::ifAutoUpdateChecksum"
   puts $rcFile "IfLeftJustifyString  $fvPref::ifLeftJustifyString"
   puts $rcFile "IfUseDesktopManager  $fvPref::ifUseManager"
   puts $rcFile "IfAutoPlotPrimary    $fvPref::ifAutoPlotPrimary"
   puts $rcFile "GraphDisplaySize     {$fvPref::graphDispSize}"
   if {$tcl_platform(platform) != "windows" } { 
      if { [winfo exists .fvpref] && [winfo ismapped [.fvpref childsite].notebook.f3.graphsize] } {
         _setGraphSize [.fvpref childsite].notebook.f3.graphsize $fvPref::graphDispSize
      }
   } else {
      if { [winfo exists .fvpref] && [winfo ismapped .fvpref.notebook.f3.graphsize] } {
         _setGraphSize .fvpref.notebook.f3.graphsize $fvPref::graphDispSize
      }
   }

#  Pan Chai: uncomment these line when XPA available in Windows
   if {$tcl_platform(platform) == "windows" } { 
      puts $rcFile "xpaLibPath           \"$fvPref::xpaLibPath\""
      puts $rcFile "ds9LibPath           \"$fvPref::ds9LibPath\""
      if [winfo exists .fvwinkeeper] {
          .fvwinkeeper updateDisplayDeviceStatus
      }
   }

   ::close $rcFile
# puts "Options saved in $_FVRC"
}

##########################################################
#############      Private Functions      ################
##########################################################


##################
#
# Setup Page: fv
#
##################

body fvPreferences::_buildPage_App { w } {
   global tcl_platform
   global g_titleFont

   checkbutton $w.fits  -text "List FITS files only"  \
         -font g_titleFont \
         -variable fvPref::ifDispFitsOnly

   if { $tcl_platform(platform)!="macintosh" } {
      checkbutton $w.mngr  -text "Show desktop manager"  \
            -variable fvPref::ifUseManager \
            -font g_titleFont \
            -command ".fvwinkeeper updateVisibility"
   }

   ::iwidgets::radiobox $w.windows -labeltext "Window Management" -labelfont g_titleFont

   $w.windows add NONE \
         -text "Leave existing windows open when opening new one" \
         -variable fvPref::winManagement -value "Keep"
   $w.windows add HIDE \
         -text "Hide existing windows when opening new one" \
         -variable fvPref::winManagement -value "Hide"
   $w.windows add CLOSE \
         -text "Close existing windows when opening new one" \
         -variable fvPref::winManagement -value "Close"

   grid $w.fits     -row 1  -column 1  -sticky w   -pady $_padYAmnt \
         -padx $_padXAmnt
   if { $tcl_platform(platform)!="macintosh" } {
      grid $w.mngr  -row 2  -column 1  -sticky w   -pady $_padYAmnt \
            -padx $_padXAmnt
   }
   grid $w.windows  -row 3  -column 1  -sticky we  -pady $_padYAmnt
   grid columnconfig $w 1 -weight 1
}

##################
#
# Setup Page: Keywords
#
##################

body fvPreferences::_buildPage_Keywords { w } {
   global g_titleFont
   checkbutton $w.hist  -text "Write history keyword after modification" \
         -font g_titleFont \
         -variable fvPref::ifWriteHisKey
   checkbutton $w.format  -text "Auto-reformat keywords"  \
         -font g_titleFont \
         -variable fvPref::ifAutoReformatKeys
   checkbutton $w.protect  -text "Protect required keywords"  \
         -font g_titleFont \
         -variable fvPref::ifProtectedKeys

   ::iwidgets::radiobox $w.chksm -labeltext "Checksum Updates" -labelfont g_titleFont

   $w.chksm add ALWAYS -text "Always update" \
         -variable fvPref::ifAutoUpdateChecksum -value 2
   $w.chksm add EXIST  -text "Update only if they exist" \
         -variable fvPref::ifAutoUpdateChecksum -value 1
   $w.chksm add NEVER  -text "Do not update" \
         -variable fvPref::ifAutoUpdateChecksum -value 0

   grid $w.hist     -row 1  -column 1  -sticky w  -pady $_padYAmnt \
         -padx $_padXAmnt
   grid $w.format   -row 2  -column 1  -sticky w  -pady $_padYAmnt \
         -padx $_padXAmnt
   grid $w.protect  -row 3  -column 1  -sticky w  -pady $_padYAmnt \
         -padx $_padXAmnt
   grid $w.chksm    -row 4  -column 1  -sticky we -pady $_padYAmnt
   grid columnconfig $w 1 -weight 1
}

##################
#
# Setup Page: Table
#
##################

body fvPreferences::_buildPage_Tables { w } {
   global g_titleFont

   ::iwidgets::radiobox $w.justify -labeltext "ASCII Column Justification" -labelfont g_titleFont

   $w.justify add LFT -text Left \
         -variable fvPref::ifLeftJustifyString -value 1
   $w.justify add RGT -text Right\
         -variable fvPref::ifLeftJustifyString -value 0

   grid $w.justify  -row 2  -column 1 -sticky ew  -pady $_padYAmnt
   grid columnconfig $w 1 -weight 1
}

##################
#
# Setup Page: Graph
#
##################

body fvPreferences::_buildPage_Graphs { w } {
   global g_hasSAOtng g_hasDS9
   global g_titleFont

   # Auto Plot Primary
   
   checkbutton $w.auto  -text "Auto-Plot Primary Image" \
         -font g_titleFont \
         -variable fvPref::ifAutoPlotPrimary

   # WCS info
   
   checkbutton $w.wcs  -text "Use WCS Info" -variable fvPref::ifWCSInfo -font g_titleFont

   # Image Viewer

   ::iwidgets::radiobox $w.viewer -labeltext "Image Viewer" -labelfont g_titleFont

   $w.viewer add DS9 -text DS9 \
         -variable fvPref::imgDisplayer -value DS9
   $w.viewer add POW -text POW \
         -variable fvPref::imgDisplayer -value POW
   $w.viewer add SAO -text SAOtng \
         -variable fvPref::imgDisplayer -value SAOTNG

   if { ! $g_hasDS9 } {
       $w.viewer buttonconfigure 0 -state disabled
   }

   if { ! $g_hasSAOtng } {
       $w.viewer buttonconfigure 2 -state disabled
   }
   # Default graph display size
   
   ::iwidgets::optionmenu $w.graphsize -labeltext "Graph Size:" -labelpos w \
         -command [code $this _setGraphSize $w.graphsize] -labelfont g_titleFont -font g_titleFont
   $w.graphsize insert 0 \
         100x100 120x120 160x160 200x200 256x256 300x300 400x400 500x500 600x600 700x700 300x400 400x300 \
         "-----" Other...
   $w.graphsize disable "-----"
   _setGraphSize $w.graphsize $fvPref::graphDispSize

   grid $w.auto       -row 1  -column 1  -sticky w  -pady $_padYAmnt \
         -padx $_padXAmnt
   grid $w.wcs        -row 2  -column 1  -sticky w  -pady $_padYAmnt \
         -padx $_padXAmnt
   grid $w.viewer     -row 3  -column 1  -sticky we -pady $_padYAmnt
   grid $w.graphsize  -row 4  -column 1  -sticky w  -pady $_padYAmnt \
         -padx $_padXAmnt

   grid columnconfig $w 1 -weight 1
}

##################
#
# Setup Page: Colors   grid columnconfig $w 1 -weight 1
#
##################

body fvPreferences::_buildPage_Colors { w } {
   global g_titleFont

   set row 1
   foreach lbl [list \
         Background: \
         Foreground: \
         "Active Background:" \
         "Active Foreground:" \
         "Check Button:"] \
         var [list \
         globalBg \
         globalFg \
         activeBg \
         activeFg \
         checkBBg] {
      label $w.clab$row -text $lbl -font g_titleFont
      button $w.cbtn$row -textvariable fvPref::${var}Color -width 7 \
            -command [code $this _changeColor $var] -font g_titleFont
      grid $w.clab$row -row $row -column 1 -sticky w  -pady 0 \
            -padx $_padXAmnt
      grid $w.cbtn$row -row $row -column 2 -sticky w  -pady 0 \
            -padx 0
      incr row
   }
   grid columnconfigure $w 2 -weight 1
}

##################
#
# Setup Page: Library Path
# Pan Chai: this function will not be called until XPA library is available in Windows
##################

body fvPreferences::_buildPage_Library { w } {
   global g_titleFont

   label $w.label -text "External Library Path" -font g_titleFont
   grid $w.label -row 1 -column 1 -sticky w  -pady 0 -padx $_padXAmnt

   label $w.label1 -text "XPA library: "
   entry $w.entry1 -text "" -width 40 -textvariable ::fvPref::xpaLibPath -bg white
   grid $w.label1 -row 3 -column 1 -sticky w  -pady 0 -padx $_padXAmnt
   grid $w.entry1 -row 3 -column 2 -sticky w  -pady 0 -padx $_padXAmnt -columnspan 3

   label $w.label2 -text "DS9 library: "
   entry $w.entry2 -text "" -width 40 -textvariable ::fvPref::ds9LibPath -bg white
   grid $w.label2 -row 4 -column 1 -sticky w  -pady 0 -padx $_padXAmnt
   grid $w.entry2 -row 4 -column 2 -sticky w  -pady 0 -padx $_padXAmnt -columnspan 3

   grid columnconfigure $w 2 -weight 1
}

body fvPreferences::_setGraphSize { menu {val ""} } {
   if { $val=="" } {
      # Update parameter for menu selection

      set value [$menu get]
      if { $value=="Other..." } {
         set val [_getGraphSize]
         _setGraphSize $menu $val
      } else {
         set fvPref::graphDispSize [split $value x]
      }

   } else {

      # Set menu selection to supplied value
      set value [join $val x]
      if { [catch {$menu index $value}] } {
         $menu insert 0 $value
      }
      $menu select $value
      set fvPref::graphDispSize $val

   }
}


body fvPreferences::_getGraphSize { } {
   global g_titleFont

   ::iwidgets::dialogshell .gSize -title "fv: Graph Size" -modality "application" 

   .gSize add OK     -text Ok      -command {.gSize deactivate 1} -font g_titleFont
   .gSize add CANCEL -text Cancel  -command {.gSize deactivate 0} -font g_titleFont
   .gSize default OK

   set w [.gSize childsite]

   label $w.wid -text "Width:" -font g_titleFont
   label $w.hgt -text "Height:" -font g_titleFont
   entry $w.wide -font g_titleFont
   entry $w.hgte -font g_titleFont

   $w.wide insert 0 [lindex $fvPref::graphDispSize 0]
   $w.hgte insert 0 [lindex $fvPref::graphDispSize 1]

   grid $w.wid   -row 1  -column 1  -padx 3 -pady 3 -sticky e
   grid $w.hgt   -row 2  -column 1  -padx 3 -pady 3 -sticky e
   grid $w.wide  -row 1  -column 2  -padx 3 -pady 3 -sticky w
   grid $w.hgte  -row 2  -column 2  -padx 3 -pady 3 -sticky w

   set result [.gSize activate]
   set width  [$w.wide get]
   set height [$w.hgte get]
   destroy .gSize

   if { $result!="" && $result } {
      if { $width > 19 && $height > 19 && $width < 2001 && $height < 2001 } {
         return [list $width $height]
      }
      tk_messageBox -type ok -icon error -parent .fvpref -title "Bad Size" \
            -message "The graph size entered does not fall into the range\
            20-2000.  Reverting to previous value."
   }
   return $fvPref::graphDispSize
}



body fvPreferences::_changeColor {type} {
   
   switch $type {
      "globalBg" { set currColor $fvPref::globalBgColor }
      "globalFg" { set currColor $fvPref::globalFgColor }
      "activeBg" { set currColor $fvPref::activeBgColor }
      "activeFg" { set currColor $fvPref::activeFgColor }
      "checkBBg" { set currColor $fvPref::checkBBgColor }
      default    { set currColor "black" }
   }

   set tmpColor [tk_chooseColor -initialcolor $currColor]
   if { $tmpColor == "" } return 
   
   switch $type {
      "globalBg" {
         option add *Background          $tmpColor
         option add *HighlightBackground $tmpColor
         set fvPref::globalBgColor       $tmpColor
      }
      "globalFg" {
         option add *Foreground          $tmpColor
         set fvPref::globalFgColor       $tmpColor
      }
      "activeFg" {
         option add *activeForeground    $tmpColor
         set fvPref::activeFgColor       $tmpColor
      }
      "activeBg" {
         option add *activeBackground    $tmpColor
         set fvPref::activeBgColor       $tmpColor
      }
      "checkBBg" {
         option add *selectColor         $tmpColor
         set fvPref::checkBBgColor       $tmpColor
      }
      default {error "Unknown color type"}
   }

   # find all the windows and change their color
   set tmpWins [winfo children .]
   foreach i $tmpWins {
      _changeWindowColors $i $type
   }
}


body fvPreferences::_changeWindowColors {win type} {

   switch $type {
      "globalBg" {
         catch {$win configure -background $fvPref::globalBgColor}
         catch {$win configure -highlightbackground $fvPref::globalBgColor}
         if { [winfo class $win] == "Checkbutton" } {
            catch {$win configure -activebackground $fvPref::globalBgColor}
         }
      }
      "globalFg" {
         Notebook:config $note -textFg $fvPref::globalFgColor
         catch {$win configure -foreground $fvPref::globalFgColor}	    
      }
      "activeFg" {
         catch {$win configure -activeforeground $fvPref::activeFgColor}
      }
      "activeBg" {
         catch {$win configure -activebackground $fvPref::activeBgColor}
      }
      "checkBBg" {
         catch {$win configure -selectcolor $fvPref::checkBBgColor}
      }
      default {error "Unknown bg type"}
   }
   
   set tmpChildren [winfo children $win]
   if { $tmpChildren == "" } {
      return
   } else {
      foreach i $tmpChildren {
         _changeWindowColors $i $type 
      }
   }
}
