set helpMsg "Usage: POWplot ?-cmap code? ?-display client? ?fitsimage?
 code: 0 Chose the best colormap
       1 Force to install private pseudo colormap
       2 Force to use truecolor colormap (if don't have it will crash!) 
       3 Force to use screen default colormap
 client: XPA entry point of form IP_Address:portNumber, pointing to
         a remote POW session to which plot commands should be sent"


global g_backupDir tcl_platform

if { $tcl_platform(platform) == "windows" } {
   set fvTmp "fv_tmp"
} elseif { $tcl_platform(platform) == "macintosh" } {
   set fvTmp "fv_temp_folder"
} else {
   set fvTmp ".fvtmp"
}

set g_backupDir $env(HOME)/$fvTmp

if ![file exists $g_backupDir] {
   file mkdir $g_backupDir
   file attributes $g_backupDir -permissions 00775
}

set ppFitsDataType(8) 0
set ppFitsDataType(16) 1
set ppFitsDataType(32) 2
set ppFitsDataType(-32) 3
set ppFitsDataType(-64) 4

set instPos [lsearch $argv "-cmap"]
set colorCode 0

# if install colormap
if { $instPos != -1} {
    if { [catch {set colorCode [lindex $argv [expr $instPos+1]]}] == 1} {
	puts $helpMsg
	exit
    }
    if { ($colorCode != 0) && ($colorCode != 1) && ($colorCode != 2) \
	&& ($colorCode != 3) } {
	puts $helpMsg
	exit
    }	
    set argv [lreplace $argv $instPos [expr $instPos+1]]
    incr argc -2 
}

wm withdraw .
update idletask

#
# Load libraries
#

set POWLIB "$env(POW_LIBRARY)/.."
set env(POW_HELPDIR) $env(POW_LIBRARY)

package require Itcl
package require Itk
package require Iwidgets

load [glob $POWLIB/libfitstcl.{s\[ol\]*,dylib}]
load [glob $POWLIB/libpow.{s\[ol\]*,dylib}]

# Look for client/server flag

set instPos [lsearch $argv "-display"]
if { $instPos != -1 } {
   set client [lindex $argv [expr $instPos+1]]
   set argv [lreplace $argv $instPos [expr $instPos+1]]
   incr argc -2
   ::powCmds::remote $client
} else {
   # None specified, so ask POW for default value
   set client [::powCmds::remote]
}

if ![info exists g_titleFont] {
    if { $tcl_platform(platform) == "windows" } {
       set isWin 1
       set isMac 0
       font create g_titleFont -family Arial             -size -12
       font create g_notebookTitleFont -family Arial     -size -14
    } elseif { $tcl_platform(platform) == "macintosh" } {
       set isWin 0
       set isMac 1
       font create g_titleFont -family system    -size -12
       font create g_notebookTitleFont -family system -size -14
    } else {
       set isWin 0
       set isMac 0
       font create g_titleFont -family Helvetica -size -12 -weight bold
       font create g_notebookTitleFont -family Helvetica -size -14 -weight bold
    }

    font create g_entryFont -family Courier -size -12
    set g_charPix      [font measure g_entryFont m]

}

set globalBgColor        #cccccc
set globalFgColor        black
set activeBgColor        #eeeeee
set activeFgColor        black
set checkBBgColor        #ff3366

option add *Background          $globalBgColor
option add *Foreground          $globalFgColor
option add *HighlightBackground $globalBgColor
option add *activeForeground    $activeFgColor
option add *activeBackground    $activeBgColor
option add *selectForeground    $activeFgColor
option add *selectBackground    $activeBgColor
option add *selectColor         $checkBBgColor

::powCmds::init 30 $colorCode

foreach filename $argv {
    regsub -all {[^a-zA-Z0-9.]} [file tail $filename] "_" gname
    set filehandle [fits open $filename 0]
    set imghandle [$filehandle load image]
    set dims [$filehandle info imgdim]
    set n1 [lindex $dims 0]
    if { [llength $dims]==1 } {
       set n2 1
    } else {
       set n2 [lindex $dims 1]
    }
    set data_type [lindex [lindex [$filehandle get keyword BITPIX] 0] 1]
    set data_type $ppFitsDataType($data_type)
    if { ([catch {$filehandle get keyword BZERO}] == 0) ||
         ([catch {$filehandle get keyword BSCALE}] == 0) } {
        set data_type 4
    }

#    powCreateData ${gname}_data $imghandle $data_type [expr $n1 * $n2] 0

    set lstData [list $imghandle $data_type [list $n1 $n2] ]
    ::powCmds::array $lstData ${gname}_data PTR $tcl_platform(byteOrder)

    # Try to get WCS (or scaling) information

    set x0 1
    set y0 1
    set xinc 1
    set yinc 1
    set x_label ""
    set y_label ""
    set x_unit  "pixels"
    set y_unit  "pixels"

    set wcsinfo [$filehandle get wcs -m]
    if { [lindex $wcsinfo 4] != "none" } {
       set x_label [lindex [lindex $wcsinfo 3] 0]
       set y_label [lindex [lindex $wcsinfo 3] 1]
       set x_unit "NULL"
       set y_unit "NULL"
    }

#    powCreateImage ${gname}_img ${gname}_data \
#      0 0 $n1 $n2 $x0 $xinc $y0 $yinc pixels pixels counts
#    powCreateGraph $gname NULL ${gname}_img \
#      $x_unit $y_unit $x_label $y_label 300 300

    ::powCmds::wcs ${gname}_img $wcsinfo
    ::powCmds::create image ${gname}_img ${gname}_data $n1 $n2

    ::powCmds::graph -name ${gname}_img \
          xlabel $x_label \
          ylabel $y_label \
          xunits $x_unit \
          yunits $y_unit
    ::powCmds::create graph ${gname}_img NULL ${gname}_img

    $filehandle close
}

bind . <<powExit>> {
    set fileList [eval glob -directory $g_backupDir -nocomplain "*" ]
    foreach name $fileList {
        file delete -force $name
    }
    exit
}

if { $client != "" } {
   exit
}
